package java.util.jar;

import java.io.FilterInputStream;
import java.util.Iterator;
import java.io.DataOutputStream;
import java.io.OutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.HashMap;
import java.util.Map;

public class Manifest implements Cloneable
{
    private final Attributes attr;
    private final Map<String, Attributes> entries;
    private final JarVerifier jv;
    
    public Manifest() {
        this.attr = new Attributes();
        this.entries = new HashMap();
        this.jv = null;
    }
    
    public Manifest(InputStream inputStream) throws IOException {
        this(null, inputStream);
    }
    
    Manifest(JarVerifier jv, InputStream inputStream) throws IOException {
        this.attr = new Attributes();
        this.entries = new HashMap();
        this.read(inputStream);
        this.jv = jv;
    }
    
    public Manifest(Manifest manifest) {
        this.attr = new Attributes();
        this.entries = new HashMap();
        this.attr.putAll(manifest.getMainAttributes());
        this.entries.putAll(manifest.getEntries());
        this.jv = manifest.jv;
    }
    
    public Attributes getMainAttributes() {
        return this.attr;
    }
    
    public Map<String, Attributes> getEntries() {
        return this.entries;
    }
    
    public Attributes getAttributes(String s) {
        return (Attributes)this.getEntries().get(s);
    }
    
    Attributes getTrustedAttributes(String str) {
        Attributes attributes = this.getAttributes(str);
        if (attributes != null && this.jv != null && !this.jv.isTrustedManifestEntry(str)) {
            throw new SecurityException("Untrusted manifest entry: " + str);
        }
        return attributes;
    }
    
    public void clear() {
        this.attr.clear();
        this.entries.clear();
    }
    
    public void write(OutputStream out) throws IOException {
        DataOutputStream dataOutputStream = new DataOutputStream(out);
        this.attr.writeMain(dataOutputStream);
        for (Map.Entry entry : this.entries.entrySet()) {
            StringBuffer o = new StringBuffer("Name: ");
            String str = (Object)entry.getKey();
            if (str != null) {
                byte[] bytes = str.getBytes("UTF8");
                str = new String(bytes, 0, 0, bytes.length);
            }
            o.append(str);
            o.append("\r\n");
            make72Safe(o);
            dataOutputStream.writeBytes(o.toString());
            ((Attributes)entry.getValue()).write(dataOutputStream);
        }
        dataOutputStream.flush();
    }
    
    static void make72Safe(StringBuffer sb) {
        int length = sb.length();
        if (length > 72) {
            for (int i = 70; i < length - 2; i += 72, length += 3) {
                sb.insert(i, "\r\n ");
            }
        }
    }
    
    public void read(InputStream inputStream) throws IOException {
        Manifest.FastInputStream fastInputStream = new FastInputStream(inputStream);
        byte[] array = new byte[512];
        this.attr.read(fastInputStream, array);
        int n = 0;
        int n2 = 0;
        int max = 2;
        Object name = null;
        int n3 = 1;
        byte[] array2 = null;
        int line;
        while ((line = fastInputStream.readLine(array)) != -1) {
            if (array[--line] != 10) {
                throw new IOException("manifest line too long");
            }
            if (line > 0 && array[line - 1] == 13) {
                --line;
            }
            if (line == 0 && n3 != 0) {
                continue;
            }
            n3 = 0;
            if (name == null) {
                name = this.parseName(array, line);
                if (name == null) {
                    throw new IOException("invalid manifest format");
                }
                if (fastInputStream.peek() == 32) {
                    array2 = new byte[line - 6];
                    System.arraycopy(array, 6, array2, 0, line - 6);
                    continue;
                }
            }
            else {
                byte[] bytes = new byte[array2.length + line - 1];
                System.arraycopy(array2, 0, bytes, 0, array2.length);
                System.arraycopy(array, 1, bytes, array2.length, line - 1);
                if (fastInputStream.peek() == 32) {
                    array2 = bytes;
                    continue;
                }
                name = new String(bytes, 0, bytes.length, "UTF8");
                array2 = null;
            }
            Attributes attributes = this.getAttributes(name);
            if (attributes == null) {
                attributes = new Attributes(max);
                this.entries.put(name, attributes);
            }
            attributes.read(fastInputStream, array);
            ++n;
            n2 += attributes.size();
            max = Math.max(2, n2 / n);
            name = null;
            n3 = 1;
        }
    }
    
    private String parseName(byte[] bytes, int n) {
        if (this.toLower((int)bytes[0]) == 110 && this.toLower((int)bytes[1]) == 97 && this.toLower((int)bytes[2]) == 109 && this.toLower((int)bytes[3]) == 101 && bytes[4] == 58 && bytes[5] == 32) {
            try {
                return new String(bytes, 6, n - 6, "UTF8");
            }
            catch (Exception ex) {}
        }
        return null;
    }
    
    private int toLower(int n) {
        return (n >= 65 && n <= 90) ? (97 + (n - 65)) : n;
    }
    
    @Override
    public boolean equals(Object o) {
        if ((o instanceof Manifest)) {
            Manifest manifest = (Manifest)o;
            return this.attr.equals(manifest.getMainAttributes()) && this.entries.equals(manifest.getEntries());
        }
        return false;
    }
    
    @Override
    public int hashCode() {
        return this.attr.hashCode() + this.entries.hashCode();
    }
    
    public Object clone() {
        return new Manifest(this);
    }
    
    static class FastInputStream extends FilterInputStream
    {
        private byte[] buf;
        private int count;
        private int pos;
        
        FastInputStream(InputStream inputStream) {
            this(inputStream, 8192);
        }
        
        FastInputStream(InputStream in, int n) {
            super(in);
            this.count = 0;
            this.pos = 0;
            this.buf = new byte[n];
        }
        
        @Override
        public int read() throws IOException {
            if (this.pos >= this.count) {
                this.fill();
                if (this.pos >= this.count) {
                    return -1;
                }
            }
            return Byte.toUnsignedInt(this.buf[this.pos++]);
        }
        
        @Override
        public int read(byte[] b, int off, int len) throws IOException {
            int n = this.count - this.pos;
            if (n <= 0) {
                if (len >= this.buf.length) {
                    return this.in.read(b, off, len);
                }
                this.fill();
                n = this.count - this.pos;
                if (n <= 0) {
                    return -1;
                }
            }
            if (len > n) {
                len = n;
            }
            System.arraycopy(this.buf, this.pos, b, off, len);
            this.pos += len;
            return len;
        }
        
        public int readLine(byte[] array, int n, int n2) throws IOException {
            byte[] buf = this.buf;
            int i = 0;
            while (i < n2) {
                int n3 = this.count - this.pos;
                if (n3 <= 0) {
                    this.fill();
                    n3 = this.count - this.pos;
                    if (n3 <= 0) {
                        return -1;
                    }
                }
                int n4 = n2 - i;
                if (n4 > n3) {
                    n4 = n3;
                }
                int pos = this.pos;
                while (pos < pos + n4 && buf[pos++] != 10) {}
                int n5 = pos - this.pos;
                System.arraycopy(buf, this.pos, array, n, n5);
                n += n5;
                i += n5;
                this.pos = pos;
                if (buf[pos - 1] == 10) {
                    break;
                }
                continue;
            }
            return i;
        }
        
        public byte peek() throws IOException {
            if (this.pos == this.count) {
                this.fill();
            }
            if (this.pos == this.count) {
                return -1;
            }
            return this.buf[this.pos];
        }
        
        public int readLine(byte[] array) throws IOException {
            return this.readLine(array, 0, array.length);
        }
        
        @Override
        public long skip(long n) throws IOException {
            if (n <= 0L) {
                return 0L;
            }
            long n2 = (long)(this.count - this.pos);
            if (n2 <= 0L) {
                return this.in.skip(n);
            }
            if (n > n2) {
                n = n2;
            }
            this.pos = (int)((long)this.pos + n);
            return n;
        }
        
        @Override
        public int available() throws IOException {
            return this.count - this.pos + this.in.available();
        }
        
        @Override
        public void close() throws IOException {
            if (this.in != null) {
                this.in.close();
                this.in = null;
                this.buf = null;
            }
        }
        
        private void fill() throws IOException {
            int n = 0;
            this.pos = n;
            this.count = n;
            int read = this.in.read(this.buf, 0, this.buf.length);
            if (read > 0) {
                this.count = read;
            }
        }
    }
}
